import React, { FC, useCallback, useEffect, useState } from 'react';
import { View, Text, Image, TouchableOpacity, Linking, Dimensions, TextInput } from 'react-native';
import { FlashList } from '@shopify/flash-list';
import { CommonActions, useFocusEffect, useNavigation } from '@react-navigation/native';
import { styles } from './styles';

import { NAVIGATION_PAGES } from 'src/types';
import { StoreType, storage } from 'src/storage';

import AddImgSvg from 'assets/icons/travels-screens/add-img.svg';
import { Header, Input, PageWrapper } from 'src/components';
import { Colors } from 'src/theme';

import SearchIcon from 'assets/icons/search.svg';
import CalendarIcon from 'assets/icons/events/calendar-solid.svg';
import EarthIcon from 'assets/icons/travels-section/earth.svg';
import NomadsIcon from 'assets/icons/bottom-navigation/travellers.svg';
import CalendarPlusIcon from 'assets/icons/events/calendar-plus.svg';
import ShoppingCartIcon from 'assets/icons/events/shopping-cart.svg';

const testData = [
  {
    id: '1',
    title: 'Meeting in San Clemente',
    date: 'Early February, 2025',
    location: 'San Clemente, California, USA',
    spots: '164/300',
    image: 'https://nomadmania.com/ajax/pic/27519',
    isPaid: true,
    status: null
  },
  {
    id: '2',
    title: 'Meeting in Singapore',
    date: '08 February, 2025',
    location: 'Singapore',
    spots: '250/250',
    image: 'https://nomadmania.com/ajax/pic/27520',
    isPaid: true,
    status: 'FULL'
  },
  {
    id: '3',
    title: 'Meeting in Patras',
    date: '14 February, 2025',
    location: 'Patras, Greece',
    spots: '119/200',
    image: 'https://nomadmania.com/ajax/pic/27521',
    isPaid: false,
    status: null
  },
  {
    id: '4',
    title: 'Nepal (including the MusNepal (including the MusNepal (including the Mus',
    date: 'March 27 – April 7, 2025',
    location: 'Nepal',
    spots: '71/150',
    image: 'https://nomadmania.com/ajax/pic/27522',
    isPaid: true,
    status: 'TOUR'
  },
  {
    id: '5',
    title: 'Meeting in Nicosia, Cyprus',
    date: '15 April, 2025',
    location: 'Nicosia, Cyprus',
    spots: null,
    image: 'https://nomadmania.com/ajax/pic/27523',
    isPaid: false,
    status: null
  }
];

const renderEventCard = ({ item }: { item: (typeof testData)[0] }) => {
  return (
    <TouchableOpacity
      style={[styles.card, item.status ? { backgroundColor: Colors.FILL_LIGHT } : {}]}
    >
      <View style={styles.imageWrapper}>
        <Image source={{ uri: item.image }} style={styles.image} />
        {item.isPaid && (
          <View style={styles.iconOverlay}>
            <ShoppingCartIcon fill={Colors.WHITE} width={12} />
          </View>
        )}
      </View>

      <View style={styles.info}>
        <Text style={styles.title} numberOfLines={1}>
          {item.title}
        </Text>

        <View style={styles.row}>
          <CalendarIcon fill={Colors.DARK_BLUE} height={14} width={14} />
          <Text style={styles.dateAndLocation} numberOfLines={1}>
            {item.date}
          </Text>
        </View>

        <View style={styles.row}>
          <EarthIcon fill={Colors.DARK_BLUE} height={14} width={14} />
          <Text style={styles.dateAndLocation} numberOfLines={1}>
            {item.location}
          </Text>
        </View>

        {item.spots && (
          <View style={styles.row}>
            <NomadsIcon fill={Colors.DARK_BLUE} height={14} width={14} />
            <Text style={styles.dateAndLocation} numberOfLines={1}>
              <Text style={{ fontWeight: '700' }}>{item.spots}</Text> Spots Available
            </Text>
          </View>
        )}
      </View>

      {item.status && (
        <View
          style={[styles.statusBadge, item.status === 'FULL' ? styles.fullBadge : styles.tourBadge]}
        >
          <View style={styles.rotatedContainer}>
            <Text style={styles.statusText}>{item.status}</Text>
          </View>
        </View>
      )}
    </TouchableOpacity>
  );
};

const EventsScreen = () => {
  const token = (storage.get('token', StoreType.STRING) as string) ?? null;
  const navigation = useNavigation();
  const [searchQuery, setSearchQuery] = useState('');
  const [filteredData, setFilteredData] = useState(testData);

  const handleSearch = (text: string) => {
    if (text) {
      const searchData =
        testData?.filter((item: any) => {
          const itemData = item.title ? item.title.toLowerCase() : ''.toLowerCase();
          const textData = text.toLowerCase();
          return itemData.indexOf(textData) > -1;
        }) ?? [];
      setFilteredData(searchData);
      setSearchQuery(text);
    } else {
      setFilteredData(testData);
      setSearchQuery(text);
    }
  };

  return (
    <PageWrapper>
      <Header
        label="Events"
        rightElement={
          <TouchableOpacity
            onPress={() => navigation.navigate(NAVIGATION_PAGES.EVENT as never)}
            style={{ width: 30 }}
          >
            <CalendarPlusIcon fill={Colors.DARK_BLUE} />
          </TouchableOpacity>
        }
      />

      <View style={styles.searchContainer}>
        <Input
          inputMode={'search'}
          placeholder={'Search'}
          onChange={(text) => handleSearch(text)}
          value={searchQuery}
          icon={<SearchIcon fill={'#C8C8C8'} width={14} height={14} />}
          height={38}
        />
      </View>

      <FlashList
        data={filteredData}
        keyExtractor={(item) => item.id}
        renderItem={renderEventCard}
        estimatedItemSize={100}
        contentContainerStyle={styles.listContainer}
      />
    </PageWrapper>
  );
};

export default EventsScreen;
